###########################################################
#
# Licensed Material - Property of IBM
#
# BogoSec - Source Code Security Quality Metric Calculator
#
# (C) Copyright IBM Corp. 2004
#
# HISTORY:
#	07/2004 originated by Tony Petz (petz.agoston@gmail.com)
#	07/2004 Dustin Kirkland (dustin.kirkland@us.ibm.com) - consistency improvements
#	01/2005 Loulwa Salem (loulwa@us.ibm.com) - added does_compile()
#	01/2005 Loulwa Salem (loulwa@us.ibm.com) - Changed global variable to all CAPS
###########################################################

package BogoFlaw;

use strict;
use File::Temp;

my $SCANNER = "flawfinder";
my $DEFAULT_ARGS = "-SQ";
my @RESULTS = ();

sub exists() {
	`which $SCANNER`;
	my $exists = !$?;
	if (!$exists) {
		print "Warning: cannot find $SCANNER binary (please include location in PATH if you wish to use it)\n";
		return undef;
	}
	return 1;
}

# this function simply returns 1 if scanner is compiler based, and 0 otherwise.
sub does_compile() {
        return 0;
}

sub get_args() {
	return $DEFAULT_ARGS;
}

sub name() {
	return $SCANNER;
}

sub analyze($$$) {
	my ($logfile, $target, $sev_range_max) = @_;
	my $sloc = undef;

# open temp file
	open(FH, $logfile);

	foreach (<FH>) {
		if (m/^$target/) {
			chomp;
			my @temp = split(/:/);
			($temp[2], $temp[3]) = $temp[2] =~ m/^\s*\[([0-5])\]\s*(.*)/;
			$temp[2] = calc_score($temp[2],$sev_range_max);
			my $temp = join(":", @temp);
			push(@RESULTS, $temp);
		} elsif (m/^Physical Source Lines of Code/) {
# grab SLOC from scanner
			my ($tmp_sloc) = m/^.*=\s*(.*)$/;
			$sloc += $tmp_sloc;
		}
	}
# push SLOC onto front of results array
	unshift @RESULTS, $sloc;
	close(FH);
	return \@RESULTS;
}

sub calc_score($$) {
	($_, my $max) = @_;
	return $_ * ($max/5);
}

sub debug {
	foreach (@RESULTS) {
		print "$_\n";
	}
}

1;
